Module Module1


    Sub Main()
        Console.WriteLine("Console GZIP Utility version {0}", System.Reflection.Assembly.GetExecutingAssembly().GetName().Version)
        Console.WriteLine("Copyright (c) Michal A. Valasek - Altair Communications, 2005")
        Console.WriteLine()

        ' Read parameters
        If System.Environment.GetCommandLineArgs().Length <> 4 Then ShowHelp()
        Dim Command As String = System.Environment.GetCommandLineArgs()(1).ToLower()
        Dim InFileName As String = System.Environment.GetCommandLineArgs()(2)
        Dim OutFileName As String = System.Environment.GetCommandLineArgs()(3)

        ' Validate parameters
        If Not System.IO.File.Exists(InFileName) Then CrashExit("ERROR: Input file not found")
        If System.IO.File.Exists(OutFileName) Then CrashExit("ERROR: Output file already exists")

        ' Perform operation
        Select Case Command
            Case "-c", "/c"
                CompressFile(InFileName, OutFileName)
            Case "-d", "/d"
                DecompressFile(InFileName, OutFileName)
            Case Else
                CrashExit("ERROR: Incorrect command; run 'cgz -?' for help")
        End Select
    End Sub

    Sub CompressFile(ByVal InFileName As String, ByVal OutFileName As String)
        Const BufferSize As Int32 = 65536 ' 64 kB

        ' Start stopwatch
        Dim SW As New System.Diagnostics.Stopwatch()
        SW.Start()

        ' Prepare streams
        Dim InStream As New System.IO.FileStream(InFileName, IO.FileMode.Open, IO.FileAccess.Read)
        Dim OutStream As New System.IO.FileStream(OutFileName, IO.FileMode.Create, IO.FileAccess.Write)
        Dim ZipStream As New System.IO.Compression.GZipStream(OutStream, IO.Compression.CompressionMode.Compress, True)

        ' Compress
        Dim Data(BufferSize) As Byte
        Dim BytesRead As Int32
        Do
            BytesRead = InStream.Read(Data, 0, BufferSize)
            ZipStream.Write(Data, 0, BytesRead)

            ' Show progress
            Console.Write("Progress: {0:N2}%", InStream.Position / InStream.Length * 100)
            Console.CursorLeft = 0
        Loop While BytesRead = BufferSize

        ' Clean up
        SW.Stop()
        Console.WriteLine("Compression completed in {0:N2} seconds.", SW.Elapsed.TotalSeconds)
        ZipStream.Close()
        OutStream.Close()
        InStream.Close()
    End Sub

    Sub DecompressFile(ByVal InFileName As String, ByVal OutFileName As String)
        Const BufferSize As Int32 = 65536 ' 64 kB

        ' Start stopwatch
        Dim SW As New System.Diagnostics.Stopwatch()
        SW.Start()

        ' Prepare streams
        Dim InStream As New System.IO.FileStream(InFileName, IO.FileMode.Open, IO.FileAccess.Read)
        Dim OutStream As New System.IO.FileStream(OutFileName, IO.FileMode.Create, IO.FileAccess.Write)
        Dim ZipStream As New System.IO.Compression.GZipStream(InStream, IO.Compression.CompressionMode.Decompress, True)

        ' Decompress
        Dim Data(BufferSize) As Byte
        Dim BytesRead As Int32
        Do
            BytesRead = ZipStream.Read(Data, 0, BufferSize)
            OutStream.Write(Data, 0, BytesRead)

            ' Show progress
            Console.Write("Progress: {0:N2}%", InStream.Position / InStream.Length * 100)
            Console.CursorLeft = 0
        Loop While BytesRead = BufferSize

        ' Clean up
        SW.Stop()
        Console.WriteLine("Decompression completed in {0:N2} seconds.", SW.Elapsed.TotalSeconds)
        ZipStream.Close()
        OutStream.Close()
        InStream.Close()
    End Sub

    Sub CrashExit(ByVal Message As String)
        Console.WriteLine("ERROR: {0}", Message)
        System.Environment.Exit(1)
    End Sub

    Sub ShowHelp()
        Console.WriteLine("USAGE:")
        Console.WriteLine("cgz -c infile outfile")
        Console.WriteLine("  Compress file 'infile' to 'outfile'.")
        Console.WriteLine("cgz -d infile outfile")
        Console.WriteLine("  Decompress file 'infile' to 'outfile'.")
        Console.WriteLine("cgz -?")
        Console.WriteLine("  Show this help screen")
        System.Environment.Exit(0)
    End Sub

End Module
